/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                         SFS_cfg.h                        *
*                   pC/SFS CONFIGURATION                   *
***********************************************************/
#ifndef _SFS_cfg_
 #define _SFS_cfg_

#include "../../pC_OS/inc/pC_OS.h"               // only for use of pC/OS
                                                 // or your have to include ..
                                                 // .. your own definition file (types)
/*
************************************************************
*                        SFS CONFIG
************************************************************
*/
#define SFS_HANDLES         3                    // max files opened by a user (task)
#define SFS_TempDIR         1                    // use the one-level temorary current-dir feature
#define SFS_AUTO_FORMAT     1                    // auto-format during init if no valid ROOT-entry was found
#define SFS_AUTO_CLOSE      1                    // close all open files of a user automatically on SFS_CloseUser()
#define SFS_LINKS           1                    // link create & delete supported
#define SFS_REMOVEDIRTREE   1                    // delete a dir and all sub-elements

/*
************************************************************
*                    SUB-CONFIGURATION
************************************************************
*/
#define SFS_BLOCK_SIZE       128                 // 128 byte per managed block
#define SFS_ENTRIES_PER_DIR   12                 // 12 entries per dir a 16 byte + padding => 256 byte (one is lost for "..")

/*
************************************************************
*           CONSTANTS & TYPES (NEVER CHANGE THIS)
************************************************************
*/
typedef U32             SFS_LONG;                // for all variables using length or part of length of SFS
typedef U08             SFS_HANDLE;              // FileHandle is an byte
typedef U08             SFS_ATTR;                // Attribs in an byte

/*
************************************************************
*                       OS ADAPTION
************************************************************
*/
#ifdef _pC_OS_
 #define OS_USER_ID     OSTCBCur->OSTCBId        // get unique task-ID
 #define SFS_MAX_USER   3                        // use max-tasks for max-users (max OS_MAX_TASKS)
 #define SFS_MUTEX_TYPE OS_SEM                   // use a semaphore on pC/OS to test sheduling during task(s) in SFS
#else
 #define OS_USER_ID     0                        // without a OS (only user-0 possible)
 #define SFS_MAX_USER   1                        // only one user (user-0)
 #define SFS_MUTEX_TYPE U08                      // use a uchar without an OS
#endif

/***********************************************************
*                   pC/SFS memory type
************************************************************
* the SFS-ports supports following byte wise read/write
* serial devices (can be more) :
***********************************************************/

 #define SFS_MEM_MR25H40         //      MRAM      SPI   Everspin   MR25H40        4Mbit      40MHz           fully byte-wise, no page-handling, no status-polling (no WREN-cmd required on every write) (and MR20H40 - 50MHz)
// #define SFS_MEM_MR25H10         //      MRAM      SPI   Everspin   MR25H10        1Mbit      40MHz           fully byte-wise, no page-handling, no status-polling (no WREN-cmd required on every write)
// #define SFS_MEM_MR25H256        //      MRAM      SPI   Everspin   MR25H256       256kbit    40MHz           fully byte-wise, no page-handling, no status-polling (no WREN-cmd required on every write)
// #define SFS_MEM_MR25H128A       //      MRAM      SPI   Everspin   MR25H128A      128kbit    40MHz           fully byte-wise, no page-handling, no status-polling (no WREN-cmd required on every write)
//
// #define SFS_MEM_CY15B104Q       //      FRAM      SPI   Cypress    CY15B104Q      4Mbit      40MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_CY15B102Q       //      FRAM      SPI   Cypress    CY15B102Q      2Mbit      20MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_CY15B256Q       //      FRAM      SPI   Cypress    CY15B256Q      256kbit    40MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_CY15B128Q       //      FRAM      SPI   Cypress    CY15B128Q      128kbit    40MHz           fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_FM25V40         //      FRAM      SPI   Ramtron    FM25V40        4Mbit      40MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_FM25V20         //      FRAM      SPI   Ramtron    FM25V20        2Mbit      40MHz           fully byte-wise, no page-handling, no status-polling (and FM25H20)
// #define SFS_MEM_FM25V10         //      FRAM      SPI   Ramtron    FM25V10        1Mbit      40MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_FM25V05         //      FRAM      SPI   Ramtron    FM25V05        512kbit    40MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_FM25V02A        //      FRAM      SPI   Ramtron    FM25V02A       256kbit    40MHz           fully byte-wise, no page-handling, no status-polling (and FM25W256)
// #define SFS_MEM_FM25V01A        //      FRAM      SPI   Ramtron    FM25V01A       128kbit    40MHz           fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_MB85RS2MT       //      FRAM      SPI   Fujitsu    MB85RS2MT      2Mbit      25MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_MB85RS1MT       //      FRAM      SPI   Fujitsu    MB85RS1MT      1Mbit      30MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_MB85RS512       //      FRAM      SPI   Fujitsu    MB85RS512      512kbit    30MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_MB85RS256       //      FRAM      SPI   Fujitsu    MB85RS256      256kbit    30MHz           fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_MB85RS128       //      FRAM      SPI   Fujitsu    MB85RS128      128kbit    30MHz           fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_CY14B101Q2      //     nvRAM      SPI   Cypress    CY14B101Q2     1Mbit      40MHz           fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_CAT25M02        //    EEPROM      SPI   ON semi    CAT25M02       2Mbit      10MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_CAT25M01        //    EEPROM      SPI   ON semi    CAT25M01       1Mbit      10MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_CAT25512        //    EEPROM      SPI   ON semi    CAT25512       512kbit    10MHz           buffered byte-wise, 128byte page-handling, status-polling
// #define SFS_MEM_CAT25256        //    EEPROM      SPI   ON semi    CAT25256       256kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
// #define SFS_MEM_CAT25128        //    EEPROM      SPI   ON semi    CAT25128       128kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
//
// #define SFS_MEM_25AA1024        //    EEPROM      SPI   Microchip  25AA1024       1Mbit      10MHz           buffered byte-wise, 256byte page-handling, status-polling (and 25LC1024)
// #define SFS_MEM_25AA512         //    EEPROM      SPI   Microchip  25AA512        512kbit    10MHz           buffered byte-wise, 128byte page-handling, status-polling (and 25LC512)
// #define SFS_MEM_25AA256         //    EEPROM      SPI   Microchip  25AA256        256kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling (and 25LC256)
// #define SFS_MEM_25AA128         //    EEPROM      SPI   Microchip  25AA128        128kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling (and 25LC128)
//
// #define SFS_MEM_M95M02          //    EEPROM      SPI   STM        M95M02         2Mbit       5MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_M95M01          //    EEPROM      SPI   STM        M95M01         1Mbit      10MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_M95512          //    EEPROM      SPI   STM        M95512         512kbit    10MHz           buffered byte-wise, 128byte page-handling, status-polling
// #define SFS_MEM_M95256          //    EEPROM      SPI   STM        M95256         256kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
// #define SFS_MEM_M95128          //    EEPROM      SPI   STM        M95128         128kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
//
// #define SFS_MEM_AT25M02         //    EEPROM      SPI   Atmel      AT25M02        2Mbit       5MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_AT25M01         //    EEPROM      SPI   Atmel      AT25M01        1Mbit      10MHz           buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_AT25512         //    EEPROM      SPI   Atmel      AT25512        512kbit    10MHz           buffered byte-wise, 128byte page-handling, status-polling
// #define SFS_MEM_AT25256         //    EEPROM      SPI   Atmel      AT25256        256kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
// #define SFS_MEM_AT25128         //    EEPROM      SPI   Atmel      AT25128        128kbit    10MHz           buffered byte-wise,  64byte page-handling, status-polling
//
/**********************************************************/
//
// #define SFS_MEM_FM24V10         //      FRAM      I2C   Ramtron    FM24V10        1Mbit      3.4MHz          fully byte-wise, no page-handling, no status-polling
// #define SFS_MEM_FM24W256        //      FRAM      I2C   Ramtron    FM24W256       256kbit      1MHz          fully byte-wise, no page-handling, no status-polling
//    
// #define SFS_MEM_24AA1025        //    EEPROM      I2C   Microchip  25AA1024       1Mbit        1MHz          buffered byte-wise, 128byte page-handling, status-polling (and 24LC1025 / 24FC1025)
//    
// #define SFS_MEM_AT24CM02        //    EEPROM      I2C   Atmel      AT24CM02       2Mbit        1MHz          buffered byte-wise, 256byte page-handling, status-polling
// #define SFS_MEM_AT24CM01        //    EEPROM      I2C   Atmel      AT24CM01       1Mbit        1MHz          buffered byte-wise, 256byte page-handling, status-polling (and AT24C1024)
//    
// #define SFS_MEM_MB85RC256       //      FRAM      I2C   Fujitsu    MB85RC256      256kbit      1MHz          fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_CAT24M01        //    EEPROM      I2C   ON semi    CAT24M01       1Mbit        1MHz          buffered byte-wise, 256byte page-handling, status-polling
//
// #define SFS_MEM_M24M02          //    EEPROM      I2C   STM        M24M02         2Mbit        1MHz          buffered byte-wise, 256byte page-handling, status-polling
//
/**********************************************************/
//
// #define SFS_MEM_xRAM_Port       //   special adapter to use RAM or an ro-FLASH image (one of the following extra HW-types can be additional used)
//
// #define SFS_MEM_MR4A16B         //      MRAM parallel   Everspin   MR4A16B       16Mbit       35ns           fully byte-wise, no page-handling, no status-polling (no WREN-cmd required on every write)
//
// #define SFS_MEM_FM22L16         //      FRAM parallel   Ramtron    FM22L16        4Mbit       55ns           fully byte-wise, no page-handling, no status-polling
//
// #define SFS_MEM_AT28LV010       //    EEPROM parallel   Atmel      AT28LV010      1Mbit      200ns           buffered byte-wise, 128byte page-handling, status-polling
//
/**********************************************************/
//
// #define SFS_MEM_FFS_Port        //   special adapter to use SD/MMC cards via pC/FFS port (as test)
//

/*
************************************************************
* SFS LL-driver config (SPI & I2C) based on SFS memory type
************************************************************
*/
#if (defined(SFS_MEM_MR25H40) || defined(SFS_MEM_MR25H10) || defined(SFS_MEM_MR25H256) || defined(SFS_MEM_MR25H128A))
 #define SFS_MEM_HW_WREN              0        // on MRAM from Everspin not required on every write
#endif

#if (defined(SFS_MEM_MR25H40) || defined(SFS_MEM_CY15B104Q) || defined(SFS_MEM_FM25V40))
 #define SFS_MEM_HW_MEMSIZE_KB      512        // in kByte
#endif

#if (defined(SFS_MEM_CY15B102Q) || defined(SFS_MEM_FM25V20) || defined(SFS_MEM_MB85RS2MT))
 #define SFS_MEM_HW_MEMSIZE_KB      256
#endif

#if (defined(SFS_MEM_MR25H10) || defined(SFS_MEM_FM25V10) || defined(SFS_MEM_MB85RS1MT) || defined(SFS_MEM_CY14B101Q2) || defined(SFS_MEM_FM24V10))
 #define SFS_MEM_HW_MEMSIZE_KB      128
#endif

#if (defined(SFS_MEM_FM25V05) || defined(SFS_MEM_MB85RS512))
 #define SFS_MEM_HW_MEMSIZE_KB       64
#endif

#if (defined(SFS_MEM_MR25H256) || defined(SFS_MEM_CY15B256Q) || defined(SFS_MEM_FM25V02A) || defined(SFS_MEM_MB85RS256) || defined(SFS_MEM_FM24W256) || defined(SFS_MEM_MB85RC256))
 #define SFS_MEM_HW_MEMSIZE_KB       32
#endif

#if (defined(SFS_MEM_MR25H128A) || defined(SFS_MEM_CY15B128Q) || defined(SFS_MEM_FM25V01A) || defined(SFS_MEM_MB85RS128))
 #define SFS_MEM_HW_MEMSIZE_KB       16
#endif

#if (defined(SFS_MEM_CAT25M02) || defined(SFS_MEM_M95M02) || defined(SFS_MEM_AT25M02) || defined(SFS_MEM_AT24CM02) || defined(SFS_MEM_M24M02))
 #define SFS_MEM_HW_PAGESIZE        256
 #define SFS_MEM_HW_MEMSIZE_KB      256
#endif

#if (defined(SFS_MEM_CAT25M01) || defined(SFS_MEM_25AA1024) || defined(SFS_MEM_M95M01) || defined(SFS_MEM_AT25M01)  || defined(SFS_MEM_AT24CM01) || defined(SFS_MEM_CAT24M01))
 #define SFS_MEM_HW_PAGESIZE        256
 #define SFS_MEM_HW_MEMSIZE_KB      128
#endif

#if (defined(SFS_MEM_24AA1025) )
 #define SFS_MEM_HW_PAGESIZE        128
 #define SFS_MEM_HW_MEMSIZE_KB      128
#endif

#if (defined(SFS_MEM_CAT25512) || defined(SFS_MEM_25AA512) || defined(SFS_MEM_M95512) || defined(SFS_MEM_AT25512))
 #define SFS_MEM_HW_PAGESIZE        128
 #define SFS_MEM_HW_MEMSIZE_KB       64
#endif

#if (defined(SFS_MEM_CAT25256) || defined(SFS_MEM_25AA256) || defined(SFS_MEM_M95256) || defined(SFS_MEM_AT25256))
 #define SFS_MEM_HW_PAGESIZE         64
 #define SFS_MEM_HW_MEMSIZE_KB       32
#endif

#if (defined(SFS_MEM_CAT25128) || defined(SFS_MEM_25AA128) || defined(SFS_MEM_M95128) || defined(SFS_MEM_AT25128))
 #define SFS_MEM_HW_PAGESIZE         64
 #define SFS_MEM_HW_MEMSIZE_KB       16
#endif

//----- parallel xRAM interface ----

#if defined(SFS_MEM_xRAM_Port)

 #if defined(SFS_MEM_MR4A16B)
  #define SFS_MEM_HW_WREN             0
  #define SFS_MEM_HW_PAGESIZE         1
  #define SFS_MEM_HW_MEMSIZE_KB    2048
  #define SFS_MEMORY_START   0x6C000000        // MRAM location on HY-STM32F4xxCore144
 #endif
 
 #if defined(SFS_MEM_FM22L16)
  #define SFS_MEM_HW_WREN             0
  #define SFS_MEM_HW_PAGESIZE         1
  #define SFS_MEM_HW_MEMSIZE_KB     512
  #define SFS_MEMORY_START   0x60000000        // FRAM location on SAM7SExx_DevMaster
 #endif
 
 #if defined(SFS_MEM_AT28LV010)
  #define SFS_MEM_HW_WREN             1
  #define SFS_MEM_HW_PAGESIZE       128
  #define SFS_MEM_HW_MEMSIZE_KB     128
  #define SFS_MEMORY_START   0x50000000        // EE   location on SAM7SExx_DevMaster
 #endif

 #ifndef SFS_MEMORY_START                      // or use "normal" compiler known RAM
  #define SFS_MEM_HW_WREN             0
  #define SFS_MEM_HW_PAGESIZE         1
  #define SFS_MEM_HW_MEMSIZE_KB      64        // 64kB used
 #endif

#endif

//----- special interfaces ----

#if defined(SFS_MEM_FFS_Port)
 #define SFS_MEM_HW_WREN              0
 #define SFS_MEM_HW_PAGESIZE        512
 #define SFS_MEM_HW_MEMSIZE_KB     4096        // 4MB used (as test)
#endif

/**********************************************************/

#ifndef SFS_MEM_HW_PAGESIZE
 #define SFS_MEM_HW_PAGESIZE          1        // if pagesize greater 1, alltimes page-access and status-polling will be used
#endif

#ifndef SFS_MEM_HW_WREN
 #define SFS_MEM_HW_WREN              1        // required on every write (for SPI)
#endif

#ifndef SFS_MEM_HW_MEMSIZE_KB
 #warning "no type of SPI memory defined !"
#endif

#define SFS_MEMORY_SIZE      (SFS_MEM_HW_MEMSIZE_KB * 1024)

/*
************************************************************
*                    SFS lowlevel driver
************************************************************
*/
#include "../inc/SFS_LLdriver.h"

/************************* END ****************************/
#endif
