/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                SFS_Port_SPI_AT91SAM3xxx.c                *
*                AT91SAM3xxx SPI xRAM PORT                 *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#define SFS_SPI_NPCS0      PIO_PA11A_NPCS0                 // /CS0 PIO-pin
#define SFS_SPI_MISO       PIO_PA12A_MISO                  // MISO PIO-pin
#define SFS_SPI_MOSI       PIO_PA13A_MOSI                  // MOSI PIO-pin
#define SFS_SPI_SPCK       PIO_PA14A_SPCK                  // SCK  PIO-pin

/***********************************************************
 *   Atmel AT91SAM3xxx configuration
 ***********************************************************
 *   PA11 ------------------------------------- /CS     ->  (manual used)
 *   PA12 ------------------------------------- MISO    <-
 *   PA13 ------------------------------------- MISO    <-
 *   PA14 ------------------------------------- SCK     ->
 **********************************************************/

#define CPU_CRISTAL_CLK  12000000UL                        // processor crystal-clock in Hz
/*
************************************************************
*          set max possible baudrate and return this
************************************************************
*/
U32 _sfsPortSetBR(U32 maxclk)
{
    U32 s_clksrc;
    U08 i;

    s_clksrc = CPU_CRISTAL_CLK;                                            // set start-clk (cristal-clk)
    SPI->SPI_MR &= ~SPI_MR_WDRBT;                                          // clear WDRBT at calc-startup
    if((PMC->PMC_MCKR & PMC_MCKR_CSS_Msk) == PMC_MCKR_CSS_PLLA_CLK) {      // if PLL-A is enabled ..
        if((PMC->CKGR_PLLAR & CKGR_PLLAR_DIVA_Msk) > 0) {                  // .. if PLL-divisor set ..
            s_clksrc /= (PMC->CKGR_PLLAR & CKGR_PLLAR_DIVA_Msk);           // div clk with PLL-A div
        }
        s_clksrc *= (((PMC->CKGR_PLLAR & CKGR_PLLAR_MULA_Msk) >> 16) + 1); // mul clk with PLL-A multiplier
        if(PMC->PMC_MCKR & PMC_MCKR_PLLADIV2) {
            s_clksrc /= 2;
        }
    }
    s_clksrc /= (0x0001UL << ((PMC->PMC_MCKR & PMC_MCKR_PRES_Msk) >> 4));  // div clk with MCK-div
    i = 1;                                                                 // search lowest clk-div for SPI-speed (highest possible SPI-speed)
    while(((s_clksrc / i) > maxclk) && (i != 0)) i++;
    if(i != 0) SPI->SPI_CSR[0] = (SPI->SPI_CSR[0] & ~SPI_CSR_SCBR_Msk) | (i << SPI_CSR_SCBR_Pos);
    return((i)? (s_clksrc / i) : 0);                                       // set SPI-div and return the used SPI-speed in Hz
}

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    PIOA->PIO_CODR = SFS_SPI_NPCS0;
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    PIOA->PIO_SODR = SFS_SPI_NPCS0;
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while(!(SPI->SPI_SR & SPI_SR_TDRE));                   // wait up to tx-empty
    SPI->SPI_TDR = byte;                                   // write char and send
    while(!(SPI->SPI_SR & SPI_SR_RDRF));                   // wait up to rx-full (char received)
    return((U08)SPI->SPI_RDR);                             // return received char
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    PMC->PMC_WPMR         = 0x504D4300;                    // disable PMC write protect
    SPI->SPI_WPMR         = 0x53504900;                    // disable SPI write protect
    PIOA->PIO_WPMR        = 0x50494F00;                    // disable PIO write protect
    PIOA->PIO_PDR         = SFS_SPI_MISO | SFS_SPI_MOSI | SFS_SPI_SPCK;     // disable from PIO-mode
    PIOA->PIO_ABCDSR[0]  &= ~(SFS_SPI_MISO | SFS_SPI_MOSI | SFS_SPI_SPCK);  // enable peripheral-functions A
    PIOA->PIO_ABCDSR[1]  &= ~(SFS_SPI_MISO | SFS_SPI_MOSI | SFS_SPI_SPCK);  // enable peripheral-functions A
    PMC->PMC_PCER0        = (1 << ID_PIOA);                // enable clock on PIO-interface A
    PIOA->PIO_PER         = SFS_SPI_NPCS0;                 // enable GPIO of CS-pin
    PIOA->PIO_OWER        = SFS_SPI_NPCS0;                 // enable write   CS-pin
    PIOA->PIO_SODR        = SFS_SPI_NPCS0;                 // set high       CS-pin
    PIOA->PIO_PUER        = SFS_SPI_NPCS0;                 // pullup enable  CS-pin
    PIOA->PIO_OER         = SFS_SPI_NPCS0;                 // output enable  CS-pin
    PMC->PMC_PCER0        = (1 << ID_SPI);                 // enable clock on SPI-interface
    SPI->SPI_CR           = SPI_CR_SPIDIS;                 // disable SPI
    SPI->SPI_CR           = SPI_CR_SPIEN | SPI_CR_SWRST;   // enable & reset SPI
    SPI->SPI_MR           = SPI_MR_MODFDIS | SPI_MR_MSTR;  // fault-off + (fix) + master
    SPI->SPI_CSR[0]       = SPI_CSR_SCBR_Msk | SPI_CSR_BITS_8_BIT | SPI_CSR_CPOL;  // low-speed +  8bit + clk-pol (mode 3)
    SPI->SPI_CR           = SPI_CR_SPIEN;                  // enable SPI
    (void)_sfsPortSetBR(10000000);                         // set SPI-clk to max 10MHz
/*
    while(1) {
        _NOP(); _NOP(); _NOP();
        _sfsPort_CS_en();
        _sfsPort_RW(0x55);
        _sfsPort_CS_dis();
        _NOP(); _NOP(); _NOP();
    }
*/
}

/************************* END ****************************/
