/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_SPI_RISCV.c                   *
*             SiFive FE310 RISC-V SPI xRAM PORT            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

/**********************************************************
 *   FE310 RISC-V configuration
 ***********************************************************
 *   10 / GPIO2 ------------------------------ /CS     ->
 *   13 / GPIO5 ------------------------------ SCK     ->
 *   12 / GPIO4 ------------------------------ MISO    <-
 *   11 / GPIO3 ------------------------------ MOSI    ->
 **********************************************************/
#define F_CPU     320000000     // from main.c, PLL-freq.
#define SPI_Fsck   32000000     // 32MHz

#define SPI_CS_HOLD
/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
#ifdef SPI_CS_HOLD
    SPI1->csid = 0;
#else
    GPIO0->port &= ~GPIO_pin(2);
#endif
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
#ifdef SPI_CS_HOLD
    SPI1->csid = 1;
#else
    GPIO0->port |= GPIO_pin(2);
#endif
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    U32   rx;

    while(!(SPI1->rxdata & SPI_rxdata_empty));
    while(SPI1->txdata & SPI_txdata_full);
    SPI1->txdata = byte;
    while((rx = SPI1->rxdata) & SPI_rxdata_empty);
    return (U08)rx;
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08   sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    SPI1->fctrl       = 0;
    SPI1->fmt         = SPI_fmt_proto_single | SPI_fmt_endian_msb | SPI_fmt_len_s(8);
    SPI1->sckdiv      = (F_CPU / (2 * SPI_Fsck) - 1) & SPI_sckdiv_scale_mask;
    SPI1->sckmode     = SPI_sckmode_pha | SPI_sckmode_pol;
#ifdef SPI_CS_HOLD
    SPI1->csmode      = SPI_csmode_hold;
    SPI1->delay0      = SPI_delay0_cssck_s(0x01)   | SPI_delay0_sckcs_s(0x01);
    SPI1->delay1      = SPI_delay1_intercs_s(0x01) | SPI_delay1_interxfr_s(0x00);
    GPIO0->iof_sel   &= (GPIO_pin(2) | GPIO_pin(3) | GPIO_pin(4) | GPIO_pin(5));
    GPIO0->iof_en    |= (GPIO_pin(2) | GPIO_pin(3) | GPIO_pin(4) | GPIO_pin(5));
#else
    SPI1->csmode      = SPI_csmode_off;
    GPIO0->iof_sel   &= (GPIO_pin(3) | GPIO_pin(4) | GPIO_pin(5));
    GPIO0->iof_en    |= (GPIO_pin(3) | GPIO_pin(4) | GPIO_pin(5));

    // config the additional manual line /CS
    GPIO0->iof_en    &= ~GPIO_pin(2);
    GPIO0->out_xor   &= ~GPIO_pin(2);
    GPIO0->pue       |= GPIO_pin(2);
    GPIO0->port      |= GPIO_pin(2);
    GPIO0->output_en |= GPIO_pin(2);
#endif
}

/************************* END ****************************/
