/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_I2C_SAMD21.c                  *
*                   SAMD21 I2C xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#include <samd21.h>

/***********************************************************
 *   Atmel SAMD21G18 configuration
 ***********************************************************
 *
 *   PA22 --- SERCOM5.0 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PA23 --- SERCOM5.1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/

enum {
  I2C_TRANSFER_WRITE = 0,
  I2C_TRANSFER_READ  = 1,
};

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM5->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM5->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM5->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_READ;
    while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    SERCOM5->I2CM.CTRLB.reg &= ~SERCOM_I2CM_CTRLB_ACKACT;
    for(i = 0; i < (len-1); i++) {
        data[i] = SERCOM5->I2CM.DATA.reg;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    }
    SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_ACKACT;
    if(len) {
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        data[len-1] = SERCOM5->I2CM.DATA.reg;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM5->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM5->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        SERCOM5->I2CM.DATA.reg = data[i];
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM5->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM5->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        SERCOM5->I2CM.DATA.reg = value;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U32 addr = 0;

    do {
        SERCOM5->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
        while (0 == (SERCOM5->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        SERCOM5->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    }
    while (SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK);
}

/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    PM->APBCMASK.reg  |= PM_APBCMASK_SERCOM5;
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_SERCOM5_CORE;
    SERCOM5->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SWRST;
    while(SERCOM5->I2CM.SYNCBUSY.bit.SWRST) __NOP();
    SERCOM5->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SPEED(1) | SERCOM_I2CM_CTRLA_SDAHOLD(0x2) | 
                              SERCOM_I2CM_CTRLA_RUNSTDBY | SERCOM_I2CM_CTRLA_MODE_I2C_MASTER;
    SERCOM5->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_SMEN | SERCOM_I2CM_CTRLB_ACKACT;
    while(SERCOM5->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
    SERCOM5->I2CM.BAUD.reg = SERCOM_I2CM_BAUD_BAUD(11) | SERCOM_I2CM_BAUD_BAUDLOW(22);
    while(SERCOM5->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
    PORT->Group[0].PINCFG[PIN_PA22].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA23].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA22/2].reg = (PORT_PMUX_PMUXE_D | PORT_PMUX_PMUXO_D);
    SERCOM5->I2CM.CTRLA.reg |= SERCOM_I2CM_CTRLA_ENABLE;
    while(SERCOM5->I2CM.SYNCBUSY.bit.ENABLE) __NOP();
    while(!(SERCOM5->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_BUSSTATE(1))) {
        SERCOM5->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_BUSSTATE(1); 
    }
}

/************************* END ****************************/
