/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                  SFS_LLdriver_FFS_Port.c                 *
*                      lowlevel driver                     *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../inc/pC_SFS.h"

#include "../../pC_FFS/inc/MMC/FFS_Port_MMC.h"

#if (SFS_MEM_HW_PAGESIZE != 512)
 #error "SD/MMC interface 'FFS_Port' is wrong configured !"
#endif

FFS_SIZE  *FFSp;
U08        buffer[SFS_MEM_HW_PAGESIZE];
/*
************************************************************
*            INIT SPI-INTERFACE & SD/MMC storage
************************************************************
*/
U08 SFSPort_Init(void)
{
    U08 ret;

    if(FFS_NO_ERR == (ret = FFSPort_Init())) {
        if(FFS_NO_ERR == (ret = FFSPort_MediaDetect())) {
            if((FFSp = FFSPort_MediaInit()) != NULL) {
                if(FFSp->maxsectors >= (SFS_MEMORY_SIZE / SFS_MEM_HW_PAGESIZE)) ret = SFS_NO_ERR;
                else ret = SFS_PORT_ERR;
            }
        }
    }
    return ret;
}

/*
************************************************************
*                READ FROM SD/MMC storage
************************************************************
*/
U08 SFSPort_Read(U32 offs, U08 OS_HUGE *dest, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG   s, i, dest_sec = 0;
    U08        ret = SFS_NO_ERR;

    while((length > 0) && (ret == SFS_NO_ERR)) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        ret = FFSPort_ReadSector(offs / SFS_MEM_HW_PAGESIZE, buffer);
        if (ret == SFS_NO_ERR) {
            for (i = 0; i < s; i++) dest[dest_sec + i] = buffer[(offs % SFS_MEM_HW_PAGESIZE) + i];
        }
        offs     += s;
        length   -= s;
        dest_sec += s;
    }
    return(ret);
}

/*
************************************************************
*     WRITE ALL OR PAGE INTO SD/MMC storage (internal)
************************************************************
*/
U08 sfsPortWriteP(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG   i;
    U08        ret;

    if (FFS_NO_ERR == (ret = FFSPort_ReadSector(offs / SFS_MEM_HW_PAGESIZE, buffer))) {
        for (i = 0; i < length; i++) buffer[(offs % SFS_MEM_HW_PAGESIZE) + i] = source[i];
        ret = FFSPort_WriteSector(offs / SFS_MEM_HW_PAGESIZE, buffer);
    }
    return ret;
}

/*
************************************************************
*                WRITE INTO SD/MMC storage
************************************************************
*/
U08 SFSPort_Write(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG   s;
    U08        ret = SFS_NO_ERR;

    while((length > 0) && (ret == SFS_NO_ERR)) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        ret = sfsPortWriteP(offs, source, s);
        source = &source[s];
        offs   += s;
        length -= s;
    }                                                      // up to page-size and read flag
    return(ret);
}

/*
************************************************************
*       FILL ALL OR PAGE INTO SD/MMC storage (internal)
************************************************************
*/
U08 sfsPortFillP(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG   i;
    U08        ret;

    if (FFS_NO_ERR == (ret = FFSPort_ReadSector(offs / SFS_MEM_HW_PAGESIZE, buffer))) {
        for (i = 0; i < length; i++) buffer[(offs % SFS_MEM_HW_PAGESIZE) + i] = value;
        ret = FFSPort_WriteSector(offs / SFS_MEM_HW_PAGESIZE, buffer);
    }
    return ret;
}

/*
************************************************************
*                    FILL SD/MMC storage
************************************************************
*/
U08 SFSPort_Fill(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG   s;
    U08        ret = SFS_NO_ERR;

    while((length > 0) && (ret == SFS_NO_ERR)) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        ret = sfsPortFillP(offs, value, s);
        offs   += s;
        length -= s;
    }
    return(ret);
}

/************************* END ****************************/
