/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                    SFS_LLdriver_I2C.c                    *
*                    lowlevel I2C driver                   *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../inc/pC_SFS.h"



/*
************************************************************
*                INIT I2C-INTERFACE & xRAM
************************************************************
*/
U08 SFSPort_Init(void)
{
    _sfsPort_Init();
    return(SFS_NO_ERR);
}

/*
************************************************************
*                    READ FROM xRAM
************************************************************
*/
U08 SFSPort_Read(U32 offs, U08 OS_HUGE *dest, SFS_LONG length) KC51_REENTRANT
{
    return(_sfsPort_readBytes(offs, dest, length));
}

/*
************************************************************
*                    WRITE INTO xRAM
************************************************************
*/
U08 SFSPort_Write(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
    U08      returnOk = SFS_NO_ERR;
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while((length > 0) && (returnOk == SFS_NO_ERR)) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        returnOk = _sfsPort_writeBytes(offs, source, s);
        if(returnOk == SFS_NO_ERR) _sfsPort_writeReady();
        source = &source[s];
        offs   += s;
        length -= s;
    }                                                      // up to page-size and read flag
#else
    returnOk = _sfsPort_writeBytes(offs, source, length);
#endif
    return(returnOk);
}

/*
************************************************************
*                        FILL xRAM
************************************************************
*/
U08 SFSPort_Fill(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
    U08      returnOk = SFS_NO_ERR;
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while((length > 0) && (returnOk == SFS_NO_ERR)) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        returnOk = _sfsPort_fillBytes(offs, value, s);
        if(returnOk == SFS_NO_ERR) _sfsPort_writeReady();
        offs   += s;
        length -= s;
    }
#else
    returnOk = _sfsPort_fillBytes(offs, value, length);
#endif
    return(returnOk);
}

/************************* END ****************************/
