/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                SFS_LLdriver_xRAM_Linux.c                 *
*                lowlevel driver Host image                *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>


#include "../inc/pC_SFS.h"

#if !defined(SFS_MEM_xRAM_Port)
 #error "wrong config for the RAM image SFS-port!"
#endif

#define IMAGE_FILE "./SFS_IMAGE.bin"

/**********************************************************
 *   using an RAM array as memory with image-support
 *   
 *   SFS_MEMORY_START [ SFS_MEMORY_SIZE ]
 *   
 **********************************************************/
extern U08 SFS_MEMORY_START[SFS_MEMORY_SIZE];

/*
************************************************************
*                LOAD DRIVE from a real File
************************************************************
*/
S16 SFSPort_Load()
{
    static FILE *ifile;
    S32          fsize;
    size_t       bytesread=0;

    ifile = fopen(IMAGE_FILE,"rb");
    if(ifile == NULL)
        return -1;
    fseek(ifile, 0L, SEEK_END);
    fsize = ftell(ifile);
    if(fsize != SFS_MEMORY_SIZE) {
        fclose(ifile);
    	return 1;
    } else {
        rewind(ifile);
        bytesread = fread(&SFS_MEMORY_START, 1, SFS_MEMORY_SIZE, ifile);
        fclose(ifile);
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                  FLUSH DRIVE to a real File
************************************************************
*/
S16 SFS_Flush()
{
    static FILE *ofile;
    size_t       byteswritten = 0;

    ofile = fopen(IMAGE_FILE,"wb+");
    if(ofile == NULL)
        return 0;
    rewind(ofile);
    byteswritten = fwrite(&SFS_MEMORY_START, 1, SFS_MEMORY_SIZE, ofile);
    fclose(ofile);
    return(SFS_NO_ERR);
}

/************************* END ****************************/
