/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                    SFS_RemoveDirTree.c                   *
*             delete a DIR and all sub-elements            *
*                (recursive pC/SFS extension)              *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../inc/pC_SFS.h"

#if (SFS_LINKS != 0)
 #warning "this extension can't handle links from outside of this sub-tree!"
#endif

U08 SFSRemoveTree(U08 user, SFS_ENTRY *orgDir, U08 *cntLink);

/*
************************************************************
*      REMOVE ALL SUB-ELEMENTS (recursive) (internal)      *
************************************************************
*/
U08 SFSRemoveTree(U08 user, SFS_ENTRY *orgDir, U08 *cntLink)
{
    SFS_ENTRY   checkEntry, orgEntry;
    SFS_LONG    curr_bi, i, ptr;
    U08         returnOk, c, j, linkedE = 0;
    
    i = 1;                                                 // delete all entries in this dir (expect "..")
    while(i < ROOT_entry.attr__block_cnt) {
        returnOk = SFSPort_Read((SFS_User[user]->currentEntry.block_start * ROOT_entry.bytes_used) + (i * sizeof(SFS_ENTRY)), (U08 OS_HUGE *)&checkEntry, sizeof(SFS_ENTRY));
        if(returnOk == SFS_NO_ERR) {
            if(checkEntry.name_hash != 0) {
                c = 0;
                orgEntry = checkEntry;                     // check link-line deep
                while(((orgEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_LINK)) {
                    c++;
                    ptr = ((orgEntry.block_start & (((SFS_LONG)-1)) >> 8) * ROOT_entry.bytes_used) + ((orgEntry.block_start >> (8 * (sizeof(SFS_LONG) - 1))) * sizeof(SFS_ENTRY));
                    returnOk = SFSPort_Read(ptr, (U08 OS_HUGE *)&orgEntry, sizeof(SFS_ENTRY));
                    if(returnOk != SFS_NO_ERR) {
                        break;
                    }
                    if(((orgEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_LINKS_M) == 0) {
                        returnOk = SFS_NO_LINK;            // entry itself is not linked
                        break;
                    }
                }
                if(returnOk != SFS_NO_ERR) {
                    return(returnOk);
                }
                if(c != 0) {
                    if(*cntLink < c) *cntLink = c;
                }
                c = (checkEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_LINKS_M;
                if(c != 0) {                               // check link-cnt
                    if(*cntLink < c) *cntLink = c;
                    if(((checkEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_TYPE_M) != SFS_ATTR_DIR) {
                        linkedE = 1;
                        i++;
                        continue;                          // file is linked (on dir we step into)
                    }
                }
                switch((checkEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_TYPE_M) {
                    case (SFS_ATTR_LINK | SFS_ATTR_DIR):
                    case (SFS_ATTR_LINK | SFS_ATTR_FILE):
                        if(checkEntry.block_start != 0) {
                            ptr = ((checkEntry.block_start & (((SFS_LONG)-1) >> 8)) * ROOT_entry.bytes_used) + ((checkEntry.block_start >> (8 * (sizeof(SFS_LONG) - 1))) * sizeof(SFS_ENTRY));
                            returnOk = SFSPort_Read(ptr, (U08 OS_HUGE *)&orgEntry, sizeof(SFS_ENTRY));
                            if(returnOk == SFS_NO_ERR) {
                                j = (orgEntry.attr__block_cnt >> (8 * (sizeof(SFS_LONG) - 1))) & SFS_ATTR_LINKS_M;
                                if(j > 0) {
                                    j--;
                                    orgEntry.attr__block_cnt = (orgEntry.attr__block_cnt & ~(SFS_ATTR_LINKS_M << (8 * (sizeof(SFS_LONG) - 1)))) | (j << (8 * (sizeof(SFS_LONG) - 1)));
                                    returnOk = SFSPort_Write(ptr, (U08 OS_HUGE *)&orgEntry, sizeof(SFS_ENTRY));
                                } else {
                                    returnOk = SFS_LINK_ERR;
                                }
                            }
                        } else {
                            returnOk = SFS_NO_LINK;        // no link-PTR in Link-Entry
                        }
                        break;
                    case SFS_ATTR_DIR:
                        orgEntry = SFS_User[user]->currentEntry;
                        curr_bi  = SFS_User[user]->h_entrybi;
                        SFS_User[user]->currentEntry = checkEntry;
                        SFS_User[user]->h_entrybi    = (orgEntry.block_start & (((SFS_LONG)-1) >> 8)) | (i << (8 * (sizeof(SFS_LONG) - 1)));
                        returnOk = SFSRemoveTree(user, orgDir, cntLink);    // recursive into next sub-dir
                        SFS_User[user]->currentEntry = orgEntry;
                        SFS_User[user]->h_entrybi = curr_bi;
                        if(returnOk == SFS_LINKED) linkedE = 1;
                        if(returnOk == SFS_NO_ERR) {
                            returnOk = SFS_TMP_DIR;                         // pre-span: dir is in use (current/temp by a user)
                            for(j = 0; j < SFS_MAX_USER; j++) {
                                if(SFS_User[j] != (SFS_USER OS_HUGE *)NULL) {
                                    if(j != user) {
                                        if(SFS_User[j]->currentEntry.block_start == SFS_User[user]->currentEntry.block_start) break;
#if (SFS_TempDIR != 0)
                                        if(SFS_User[j]->TempDir.block_start == SFS_User[user]->TempDir.block_start) break;
#endif
                                    } else {
                                        if(orgDir->block_start == SFS_User[user]->currentEntry.block_start) break;
                                    }
                                }
                            }
                            if(j >= SFS_MAX_USER) {        // delete the dir blocks if not current/temp by a user
                                c = ((ROOT_entry.attr__block_cnt * sizeof(SFS_ENTRY)) / ROOT_entry.bytes_used) + (((ROOT_entry.attr__block_cnt * sizeof(SFS_ENTRY)) % ROOT_entry.bytes_used)? 1 : 0);
                                returnOk = SFSMem_Free(checkEntry.block_start, c);
                            }
                        }
                        break;
                    case SFS_ATTR_FILE:
                        returnOk = SFSFileOpend(&checkEntry);
                        if(returnOk == SFS_NO_ERR) {       // delete the file blocks if not opened by a user
                            returnOk = SFSMem_Free(checkEntry.block_start, checkEntry.attr__block_cnt & (((SFS_LONG)(-1)) >> 8));
                        }
                        break;
                    default:
                        break;
                }
                if(returnOk == SFS_NO_ERR) {
                    checkEntry.name_hash = 0;              // delete the entry itself
                    ptr = (SFS_User[user]->currentEntry.block_start * ROOT_entry.bytes_used) + (i * sizeof(SFS_ENTRY));
                    returnOk = SFSPort_Write(ptr, (U08 OS_HUGE *)&checkEntry, sizeof(checkEntry.name_hash));
                }
                if((returnOk != SFS_NO_ERR) && (returnOk != SFS_LINKED)) {
                    return(returnOk);                      // something goes wrong
                }
            }
        } else {
            break;
        }
        i++;
    }                                                      // read currentEntry itself again and check if still linked
    ptr = ((SFS_User[user]->h_entrybi & (((SFS_LONG)-1) >> 8)) * ROOT_entry.bytes_used) + ((SFS_User[user]->h_entrybi >> (8 * (sizeof(SFS_LONG) - 1))) * sizeof(SFS_ENTRY));
    returnOk = SFSPort_Read(ptr, (U08 OS_HUGE *)&checkEntry, sizeof(SFS_ENTRY));
    if(((checkEntry.attr__block_cnt & (SFS_ATTR_LINKS_M << (8 * (sizeof(SFS_LONG) - 1)))) != 0) || (linkedE != 0)) {
        returnOk = SFS_LINKED;
    }
    return(returnOk);
}

/*
************************************************************
*             REMOVE DIR AND ALL SUB-ELEMENTS              *
************************************************************
*/
U08 SFS_RemoveDirTree(U08 OS_HUGE *Name) KC51_REENTRANT
{
    SFS_ENTRY   currDir;
    SFS_LONG    i, ptr;
    U08         returnOk, user, orgLink = 0, cntLink;

    returnOk = SFSCheckUser(&user);
    if(returnOk != SFS_NO_ERR) {
        return(returnOk);
    }
    currDir = SFS_User[user]->currentEntry;                // save actual position and go into PATH
    returnOk = SFSChange_Entry(Name, 5);                   // go into dir / stay on link dir
    if(returnOk == SFS_NO_ERR) {                           // delete Entry if no sub-elements and not the ROOT
        cntLink = 0;
        do {
            returnOk = SFSRemoveTree(user, &currDir, &cntLink);
            if(orgLink == 0) orgLink = cntLink; else orgLink--;
        } while((orgLink > 0) && (returnOk == SFS_LINKED));
        if(returnOk == SFS_NO_ERR) {
            returnOk = SFS_TMP_DIR;                        // pre-span: dir is in use (current/temp by a user)
            for(i = 0; i < SFS_MAX_USER; i++) {
                if(SFS_User[i] != (SFS_USER OS_HUGE *)NULL) {
                    if(i != user) {
                        if(SFS_User[i]->currentEntry.block_start == SFS_User[user]->currentEntry.block_start) break;
#if (SFS_TempDIR != 0)
                        if(SFS_User[i]->TempDir.block_start == SFS_User[user]->TempDir.block_start) break;
#endif
                    } else {
                        if(currDir.block_start == SFS_User[user]->currentEntry.block_start) break;
                    }
                }
            }
            if(i >= SFS_MAX_USER) {                        // delete the dir blocks if not current/temp by a user
                ptr = ((ROOT_entry.attr__block_cnt * sizeof(SFS_ENTRY)) / ROOT_entry.bytes_used) + (((ROOT_entry.attr__block_cnt * sizeof(SFS_ENTRY)) % ROOT_entry.bytes_used)? 1 : 0);
                returnOk = SFSMem_Free(SFS_User[user]->currentEntry.block_start, ptr);
            }
            if(returnOk == SFS_NO_ERR) {
                SFS_User[user]->currentEntry.name_hash = 0;  // delete the entry itself
                ptr = ((SFS_User[user]->h_entrybi & (((SFS_LONG)-1) >> 8)) * ROOT_entry.bytes_used) + ((SFS_User[user]->h_entrybi >> (8 * (sizeof(SFS_LONG) - 1))) * sizeof(SFS_ENTRY));
                returnOk = SFSPort_Write(ptr, (U08 OS_HUGE *)&SFS_User[user]->currentEntry, sizeof(SFS_User[user]->currentEntry.name_hash));
            }
        }
    }
    SFS_User[user]->currentEntry = currDir;                // restore saved position
    SFS_User[user]->ErrNo = SFSMutex_Put(&SFS_mutex);
    return(returnOk);
}

/************************* END ****************************/
