/***********************************************************
*                       pC/SFS 1.12b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                 SFS_Port_SPI_STM32F1xx.c                 *
*                  STM32F1xx SPI xRAM PORT                 *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include "../../../inc/STM32/library/inc/stm32f10x_rcc.h"
#include "../../../inc/STM32/library/inc/stm32f10x_spi.h"
#include "../../../inc/STM32/library/inc/stm32f10x_gpio.h"

/**********************************************************
 *   ST STM32F1xx configuration
 ***********************************************************
 *   PA4 ------------------------------------- /CS     ->  (manual used)
 *   PA5 ------------------------------------- SCK     ->
 *   PA6 ------------------------------------- MISO    <-
 *   PA7 ------------------------------------- MOSI    ->
 **********************************************************/

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    GPIO_ResetBits(GPIOA,GPIO_Pin_4);
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    GPIO_SetBits(GPIOA,GPIO_Pin_4);
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    SPI_SendData(SPI1, byte);
    while (SPI_GetFlagStatus(SPI1, SPI_FLAG_TXE) == RESET) _NOP();
    while (SPI_GetFlagStatus(SPI1, SPI_FLAG_RXNE) == RESET) _NOP(); 
    return SPI_ReceiveData(SPI1);
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    SPI_InitTypeDef  SPI_InitStruct;
    GPIO_InitTypeDef GPIO_InitStructure;

    /* GPIOA Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    /* Configure PA.05, PA.06, PA.07 pins */
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Configure PA.04 pin: out pushpull mode */
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Configure SPI1 */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    SPI_DeInit(SPI1);
    SPI_InitStruct.SPI_Direction         = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStruct.SPI_DataSize          = SPI_DataSize_8b; 
    SPI_InitStruct.SPI_Mode              = SPI_Mode_Master;
    SPI_InitStruct.SPI_CPOL              = SPI_CPOL_Low;
    SPI_InitStruct.SPI_CPHA              = SPI_CPHA_1Edge;
    SPI_InitStruct.SPI_NSS               = SPI_NSS_Soft;
    SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;
    SPI_InitStruct.SPI_FirstBit          = SPI_FirstBit_MSB;
    SPI_InitStruct.SPI_CRCPolynomial     = 7;
    SPI_Init(SPI1, &SPI_InitStruct);
    SPI_Cmd(SPI1, ENABLE);
}

/************************* END ****************************/
